"""
FritzCall:

Gives access to recent phone calls: incoming, outgoing and
missed ones.

Utility module based on FritzConnection.
Author: Klaus Bremer
"""


import argparse
import os

# tiny hack to run this as a package but also from the command line. In
# the latter case ValueError is raised from python 2.7,
# SystemError from Python 3.5
# ImportError from Python 3.6
try:
    from . import fritzconnection
except (ValueError, SystemError, ImportError):
    import fritzconnection

from datetime import datetime, timedelta
from lxml import etree

__version__ = '0.7.0'

# calltypes
ALL_CALL_TYPES = 0
RECEIVED_CALL_TYPE = 1
MISSED_CALL_TYPE = 2
OUT_CALL_TYPE = 3

CALL_TYPES = {
    RECEIVED_CALL_TYPE: 'in',
    MISSED_CALL_TYPE: 'missed',
    OUT_CALL_TYPE: 'out',
}

# access requires password
ONTEL_SERVICE = 'X_AVM-DE_OnTel'

# version-access:
def get_version():
    return __version__


class FritzCall(object):
    """
    Gives access to lists of recent phone calls: incoming, outgoing and
    missed ones.
    """

    def __init__(self,
                 fc=None,
                 address=fritzconnection.FRITZ_IP_ADDRESS,
                 port=fritzconnection.FRITZ_TCP_PORT,
                 user=fritzconnection.FRITZ_USERNAME,
                 password=''):
        super(FritzCall, self).__init__()
        if fc is None:
            fc = fritzconnection.FritzConnection(address, port, user, password)
        self.fc = fc
        self.calls = None

    def _get_calllist_iterator(self):
        result = self.fc.call_action(ONTEL_SERVICE, 'GetCallList')
        url = result['NewCallListURL']
        return etree.parse(url).getiterator()

    def _update(self, update=True):

        def get_duration(value):
            h, m = map(int, value.strip().split(':'))
            return timedelta(minutes=m, hours=h)

        def converter(entry):
            for k, v in (('Type', int), ('Id', int), ('Port', int)):
                entry[k] = v(entry[k])
            entry['Date'] = datetime.strptime(entry['Date'], '%d.%m.%y %H:%M')
            entry['Duration'] = get_duration(entry['Duration'])
            return entry

        if not self.calls:
            update = True
        if update:
            root = self._get_calllist_iterator()
            # convert elements with content to a list of dicts
            nodes = [
                {node.tag: node.text for node in elem}
                for elem in root if len(elem)
            ]
            # convert value-types for valid nodes
            self.calls = map(converter,
                [node for node in nodes if node.get('Id')])

    # ----------------------------------
    # public api:
    # ----------------------------------

    def get_calls(self, calltype=ALL_CALL_TYPES, update=True):
        """
        Returns a list of dicts. Every dictionary represents a call.
        Keys and types are:

        Id: int
        Type: int
        Caller: str
        Called: str
        CalledNumber: str
        Name: str
        Numbertype: str
        Device: str
        Port: int
        Date: datetime
        Duration: timedelta
        Count: str
        Path: str

        The values are either of the given type or None.
        """
        self._update(update)
        if calltype == ALL_CALL_TYPES:
            return self.calls
        return [call for call in self.calls if call.get('Type')==calltype]

    def get_received_calls(self, update=True):
        """Returns a list of dicts of calls from Type 1 (in)"""
        return self.get_calls(RECEIVED_CALL_TYPE, update)

    def get_missed_calls(self, update=True):
        """Returns a list of dicts of calls from Type 2 (missed)"""
        return self.get_calls(MISSED_CALL_TYPE, update)

    def get_out_calls(self, update=True):
        """Returns a list of dicts of calls from Type 3 (out)"""
        return self.get_calls(OUT_CALL_TYPE, update)


# ---------------------------------------------------------
# terminal-output:
# ---------------------------------------------------------

def print_calls(fc, calltype, num):
    print(calltype)
    template = '{:<7}{:<16}{:<16}{:<20}{}'
    calls = list(fc.get_calls(calltype))
    stored_numbers = len(calls)
    print('Numbers in storage:', stored_numbers)
    print('Entries listed:', min(num, stored_numbers))
    print(template.format('Type', 'from', 'to', 'date/time', 'duration'))
    for n, call in enumerate(calls):
        if n >= num:
            break
        calltype = CALL_TYPES.get(call.get('Type'), '-')
        caller = call.get('Caller')
        caller = caller.split()[-1] if caller else '-'
        called = call.get('Called')
        called = called.split()[-1] if called else '-'
        date = call.get('Date')
        if date:
            date = date.strftime('%Y-%m-%d %H:%M')
        duration = call.get('Duration')
        print(template.format(calltype, caller, called, date, duration))


# ---------------------------------------------------------
# cli-section:
# ---------------------------------------------------------


def _get_cli_arguments():
    parser = argparse.ArgumentParser(description='FritzBox Hosts')
    parser.add_argument('-i', '--ip-address',
                        nargs='?', default=None, const=None,
                        dest='address',
                        help='ip-address of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_IP_ADDRESS)
    parser.add_argument('--port',
                        nargs='?', default=None, const=None,
                        dest='port',
                        help='port of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_TCP_PORT)
    parser.add_argument('-u', '--username',
                        nargs='?', default=None, const=None,
                        help='Fritzbox authentication username')
    parser.add_argument('-p', '--password',
                        nargs='?', default=None, const=None,
                        help='Fritzbox authentication password')
    parser.add_argument('-a', '--all',
                        action='store_true',
                        help='Show all calls '
                             '(default if no other options given)')
    parser.add_argument('-r', '--received',
                        action='store_true',
                        help='Show received (incoming) calls')
    parser.add_argument('-o', '--out',
                        action='store_true',
                        help='Show outgoing calls')
    parser.add_argument('-m', '--missed',
                        action='store_true',
                        help='Show missed calls')
    parser.add_argument('-n', '--num', type=int,
                        nargs='?', default=999, const=999,
                        help='max number of calls')
    args = parser.parse_args()
    return args


def _print_status(arguments):
    fc = FritzCall(address=arguments.address,
                   port=arguments.port,
                   user=arguments.username,
                   password=arguments.password)
    if arguments.received:
        calltype = RECEIVED_CALL_TYPE
    elif arguments.missed:
        calltype = MISSED_CALL_TYPE
    elif arguments.out:
        calltype = OUT_CALL_TYPE
    else:
        calltype = ALL_CALL_TYPES
    print_calls(fc, calltype, arguments.num)


def main():
    _print_status(_get_cli_arguments())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
fritzconnection.py

This is a tool to communicate with the FritzBox.
All availabel actions (aka commands) and corresponding parameters are
read from the xml-configuration files requested from the FritzBox. So
the available actions may change depending on the FritzBox model and
firmware.
The command-line interface allows the api-inspection.
The api can also be inspected by a terminal session:

>>> import fritzconnection as fc
>>> fc.print_api()

'print_api' takes the optional parameters:
    address = ip-address
    port = port number (should not change)
    user = the username
    password = password (to access tr64-services)

In most cases you have to provide the ip (in case you changed the
default or have multiple boxes i.e. for multiple WLAN access points).
Also you have to send the password to get the complete api.

License: MIT https://opensource.org/licenses/MIT
Source: https://bitbucket.org/kbr/fritzconnection
Author: Klaus Bremer
"""

__version__ = '0.8.1'

import os, argparse
import requests
from requests.auth import HTTPDigestAuth

from lxml import etree


# FritzConnection defaults:
FRITZ_IP_ADDRESS = '169.254.1.1'
FRITZ_TCP_PORT = 49000
FRITZ_IGD_DESC_FILE = 'igddesc.xml'
FRITZ_TR64_DESC_FILE = 'tr64desc.xml'
FRITZ_USERNAME = 'dslf-config'


# version-access:
def get_version():
    return __version__


class FritzConnectionException(Exception): pass
class ServiceError(FritzConnectionException): pass
class ActionError(FritzConnectionException): pass
class AuthorizationError(FritzConnectionException): pass


class FritzAction(object):
    """
    Class representing an action (aka command).
    Knows how to execute itself.
    Access to any password-protected action must require HTTP digest
    authentication.
    See: http://www.broadband-forum.org/technical/download/TR-064.pdf
    """
    header = {'soapaction': '',
              'content-type': 'text/xml',
              'charset': 'utf-8'}
    envelope = """
        <?xml version="1.0" encoding="utf-8"?>
        <s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
                    xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">%s
        </s:Envelope>
        """
    body_template = """
        <s:Body>
        <u:%(action_name)s xmlns:u="%(service_type)s">%(arguments)s
        </u:%(action_name)s>
        </s:Body>
        """
    argument_template = """
        <s:%(name)s>%(value)s</s:%(name)s>"""
    method = 'post'

    def __init__(self, service_type, control_url, action_parameters):
        self.service_type = service_type
        self.control_url = control_url
        self.name = ''
        self.arguments = {}
        self.__dict__.update(action_parameters)

    @property
    def info(self):
        return [self.arguments[argument].info for argument in self.arguments]

    def _body_builder(self, kwargs):
        """
        Helper method to construct the appropriate SOAP-body to call a
        FritzBox-Service.
        """
        p = {
            'action_name': self.name,
            'service_type': self.service_type,
            'arguments': '',
            }
        if kwargs:
            arguments = [
                self.argument_template % {'name': k, 'value': v}
                for k, v in kwargs.items()
            ]
            p['arguments'] = ''.join(arguments)
        body = self.body_template.strip() % p
        return body

    def execute(self, **kwargs):
        """
        Calls the FritzBox action and returns a dictionary with the
        arguments. Raises an AuthorizationError in case a password is
        required for this request but is missing or wrong.
        """
        headers = self.header.copy()
        headers['soapaction'] = '%s#%s' % (self.service_type, self.name)
        data = self.envelope.strip() % self._body_builder(kwargs)
        url = 'http://%s:%s%s' % (self.address, self.port, self.control_url)
        auth = None
        if self.password:
            auth=HTTPDigestAuth(self.user, self.password)
        response = requests.post(url, data=data, headers=headers, auth=auth)
        if response.status_code == 401:
            # password required but missing or wrong
            raise AuthorizationError('unauthorized request')
        # lxml needs bytes, therefore response.content (not response.text)
        result = self.parse_response(response.content)
        return result

    def parse_response(self, response):
        """
        Evaluates the action-call response from a FritzBox.
        The response is a xml byte-string.
        Returns a dictionary with the received arguments-value pairs.
        The values are converted according to the given data_types.
        TODO: boolean and signed integers data-types from tr64 responses
        """
        result = {}
        root = etree.fromstring(response)
        for argument in self.arguments.values():
            try:
                value = root.find('.//%s' % argument.name).text
            except AttributeError:
                # will happen by searching for in-parameters and by
                # parsing responses with status_code != 200
                continue
            if argument.data_type.startswith('ui'):
                try:
                    value = int(value)
                except ValueError:
                    # should not happen
                    value = None
                except TypeError:
                    # raised in case that value is None. Should also not happen.
                    value = None
            result[argument.name] = value
        return result


class FritzActionArgument(object):
    """Attribute class for arguments."""
    name = ''
    direction = ''
    data_type = ''

    @property
    def info(self):
        return (self.name, self.direction, self.data_type)


class FritzService(object):
    """Attribute class for service."""

    def __init__(self, service_type, service_id, control_url, scpd_url):
        self.service_type = service_type
        self.service_id = service_id
        self.control_url = control_url
        self.scpd_url = scpd_url
        self.actions = {}

    @property
    def name(self):
        """Returns the servicename in a backward compatible way as:
        '<name>:<number>'
        """
        last_part = self.service_id.split(':')[-1]
        return ':'.join((last_part[:-1], last_part[-1]))


class FritzXmlParser(object):
    """Base class for parsing fritzbox-xml-files."""

    def __init__(self, address, port, filename=None):
        """Loads and parses an xml-file from a FritzBox."""
        if address is None:
            source = filename
        else:
            source = 'http://{0}:{1}/{2}'.format(address, port, filename)
        tree = etree.parse(source)
        self.root = tree.getroot()
        self.namespace = etree.QName(self.root.tag).namespace

    def nodename(self, name):
        """Extends name with the xmlns-prefix to a valid nodename."""
        return etree.QName(self.root, name)


class FritzDescParser(FritzXmlParser):
    """Class for parsing desc.xml-files."""

    def get_modelname(self):
        """Returns the FritzBox model name."""
        xpath = '%s/%s' % (self.nodename('device'), self.nodename('modelName'))
        return self.root.find(xpath).text

    def get_services(self):
        """Returns a list of FritzService-objects."""
        result = []
        nodes = self.root.iterfind(
            './/ns:service', namespaces={'ns': self.namespace})
        for node in nodes:
            result.append(FritzService(
                node.find(self.nodename('serviceType')).text,
                node.find(self.nodename('serviceId')).text,
                node.find(self.nodename('controlURL')).text,
                node.find(self.nodename('SCPDURL')).text))
        return result


class FritzSCDPParser(FritzXmlParser):
    """Class for parsing SCDP.xml-files"""

    def __init__(self, address, port, service, filename=None):
        """
        Reads and parses a SCDP.xml-file from FritzBox.
        'service' is a tuple of containing:
        (serviceType, controlURL, SCPDURL)
        'service' is a FritzService object:
        """
        self.state_variables = {}
        self.service = service
        if filename is None:
            # access the FritzBox
            super(FritzSCDPParser, self).__init__(address, port,
                                                  service.scpd_url)
        else:
            # for testing read the xml-data from a file
            super(FritzSCDPParser, self).__init__(None, None, filename=filename)

    def _read_state_variables(self):
        """
        Reads the stateVariable information from the xml-file.
        The information we like to extract are name and dataType so we
        can assign them later on to FritzActionArgument-instances.
        Returns a dictionary: key:value = name:dataType
        """
        nodes = self.root.iterfind(
            './/ns:stateVariable', namespaces={'ns': self.namespace})
        for node in nodes:
            key = node.find(self.nodename('name')).text
            value = node.find(self.nodename('dataType')).text
            self.state_variables[key] = value

    def get_actions(self, action_parameters):
        """Returns a list of FritzAction instances."""
        self._read_state_variables()
        actions = []
        nodes = self.root.iterfind(
            './/ns:action', namespaces={'ns': self.namespace})
        for node in nodes:
            action = FritzAction(self.service.service_type,
                                 self.service.control_url,
                                 action_parameters)
            action.name = node.find(self.nodename('name')).text
            action.arguments = self._get_arguments(node)
            actions.append(action)
        return actions

    def _get_arguments(self, action_node):
        """
        Returns a dictionary of arguments for the given action_node.
        """
        arguments = {}
        argument_nodes = action_node.iterfind(
            r'./ns:argumentList/ns:argument', namespaces={'ns': self.namespace})
        for argument_node in argument_nodes:
            argument = self._get_argument(argument_node)
            arguments[argument.name] = argument
        return arguments

    def _get_argument(self, argument_node):
        """
        Returns a FritzActionArgument instance for the given argument_node.
        """
        argument = FritzActionArgument()
        argument.name = argument_node.find(self.nodename('name')).text
        argument.direction = argument_node.find(self.nodename('direction')).text
        rsv = argument_node.find(self.nodename('relatedStateVariable')).text
        # TODO: track malformed xml-nodes (i.e. misspelled)
        argument.data_type = self.state_variables.get(rsv, None)
        return argument


class FritzConnection(object):
    """
    FritzBox-Interface to read status-information and modify settings.
    """
    def __init__(self, address=None, port=None, user=None, password=None):
        """
        Initialisation of FritzConnection: reads all data from the box
        and also the api-description (the servicenames and according
        actionnames as well as the parameter-types) that can vary among
        models and stores these informations as instance-attributes.
        This can be an expensive operation. Because of this an instance
        of FritzConnection should be created once and reused in an
        application. All parameters are optional. But if there is more
        than one FritzBox in the network, an address (ip as string) must
        be given, otherwise it is not defined which box may respond. If
        no user is given the Environment gets checked for a
        FRITZ_USERNAME setting. If there is no entry in the environment
        the avm-default-username will be used. If no password is given
        the Environment gets checked for a FRITZ_PASSWORD setting. So
        password can be used without using configuration-files or even
        hardcoding.
        """
        if address is None:
            address = FRITZ_IP_ADDRESS
        if port is None:
            port = FRITZ_TCP_PORT
        if user is None:
            user = os.getenv('FRITZ_USERNAME', FRITZ_USERNAME)
        if password is None:
            password = os.getenv('FRITZ_PASSWORD', '')
        # The keys of the dictionary are becoming FritzAction instance
        # attributes on calling the FritzSCDPParser.get_actions() method
        # in self._read_services():
        self.action_parameters = {
            'address': address,
            'port': port,
            'user': user,
            'password': password
        }
        self.address = address
        self.port = port
        self.modelname = None
        self.services = {}
        self._read_descriptions(password)

    def __repr__(self):
        """
        Return a meaningful string if the instance gets printed or
        should represent itself.
        """
        return 'FritzConnection to model {} with ip {}'.format(
            self.modelname,
            self.address,
        )

    def _read_descriptions(self, password):
        """
        Read and evaluate the igddesc.xml file
        and the tr64desc.xml file if a password is given.
        """
        descfiles = [FRITZ_IGD_DESC_FILE]
        if password:
            descfiles.append(FRITZ_TR64_DESC_FILE)
        for descfile in descfiles:
            try:
                parser = FritzDescParser(self.address, self.port, descfile)
            except IOError:
                # failed to load a resource. Can happen on customized models
                # missing the igddesc.xml file.
                # It's save to ignore this error.
                continue
            if not self.modelname:
                self.modelname = parser.get_modelname()
            services = parser.get_services()
            self._read_services(services)

    def _read_services(self, services):
        """Get actions from services."""
        for service in services:
            parser = FritzSCDPParser(self.address, self.port, service)
            actions = parser.get_actions(self.action_parameters)
            service.actions = {action.name: action for action in actions}
            self.services[service.name] = service

    @property
    def actionnames(self):
        """
        Returns a alphabetical sorted list of tuples with all known
        service- and action-names.
        """
        actions = []
        for service_name in sorted(self.services.keys()):
            action_names = self.services[service_name].actions.keys()
            for action_name in sorted(action_names):
                actions.append((service_name, action_name))
        return actions

    def _get_action(self, service_name, action_name):
        """
        Returns an action-object (an instance of FritzAction) with the
        given action_name from the given service.
        Raises a ServiceError-Exeption in case of an unknown
        service_name and an ActionError in case of an unknown
        action_name.
        """
        try:
            service = self.services[service_name]
        except KeyError:
            raise ServiceError('Unknown Service: ' + service_name)
        try:
            action = service.actions[action_name]
        except KeyError:
            raise ActionError('Unknown Action: ' + action_name)
        return action

    def get_action_arguments(self, service_name, action_name):
        """
        Returns a list of tuples with all known arguments for the given
        service- and action-name combination. The tuples contain the
        argument-name, direction and data_type.
        """
        action = self._get_action(service_name, action_name)
        return action.info

    def call_action(self, service_name, action_name, **kwargs):
        """
        Executes the given action. Raise a KeyError on unkown actions.
        service_name can end with an identifier ':n' (with n as an
        integer) to differentiate between different services with the
        same name, like WLANConfiguration:1 or WLANConfiguration:2. In
        case the service_name does not end with an identifier the id
        ':1' will get added by default.
        """
        if not ':' in service_name:
            service_name += ':1'
        action = self._get_action(service_name, action_name)
        return action.execute(**kwargs)

    def reconnect(self):
        """
        Terminate the connection and reconnects with a new ip.
        Will raise a KeyError if this command is unknown (by any means).
        """
        self.call_action('WANIPConnection', 'ForceTermination')


# ---------------------------------------------------------
# Inspection class for cli use:
# ---------------------------------------------------------

class FritzInspection(object):

    def __init__(self, address=FRITZ_IP_ADDRESS,
                       port=FRITZ_TCP_PORT,
                       user=FRITZ_USERNAME,
                       password=''):
        self.fc = FritzConnection(address, port, user, password)

    @staticmethod
    def _get_full_servicename(servicename):
        if ':' not in servicename:
            servicename += ':1'
        return servicename

    def get_servicenames(self):
        return sorted(self.fc.services.keys())

    def get_actionnames(self, servicename):
        servicename = self._get_full_servicename(servicename)
        try:
            service = self.fc.services[servicename]
        except KeyError:
            return []
        return sorted(service.actions.keys())

    def view_header(self):
        print('\nFritzConnection:')
        print('{:<20}{}'.format('version:', get_version()))
        print('{:<20}{}'.format('model:', self.fc.modelname))

    def view_servicenames(self):
        print('Servicenames:')
        for name in self.get_servicenames():
            print('{:20}{}'.format('', name))

    def view_actionnames(self, servicename):
        print('\n{:<20}{}'.format('Servicename:', servicename))
        print('Actionnames:')
        for name in self.get_actionnames(servicename):
            print('{:20}{}'.format('', name))

    def view_actionarguments(self, servicename, actionname):
        servicename = self._get_full_servicename(servicename)
        print('\n{:<20}{}'.format('Servicename:', servicename))
        print('{:<20}{}'.format('Actionname:', actionname))
        print('Arguments:')
        self._view_arguments('{:20}{}', servicename, actionname)

    def view_servicearguments(self, servicename):
        servicename = self._get_full_servicename(servicename)
        print('\n{:<20}{}'.format('Servicename:', servicename))
        actionnames = self.get_actionnames(servicename)
        for actionname in actionnames:
            print('{:<20}{}'.format('Actionname:', actionname))
            self._view_arguments('{:24}{}', servicename, actionname)

    def _view_arguments(self, fs, servicename, actionname):
        for argument in sorted(
            self.fc.get_action_arguments(servicename, actionname)):
            print(fs.format('', argument))

    def view_complete(self):
        print('FritzBox API:')
        for servicename in self.get_servicenames():
            self.view_servicearguments(servicename)


# ---------------------------------------------------------
# terminal-output:
# ---------------------------------------------------------

def print_servicenames(address=FRITZ_IP_ADDRESS,
                       port=FRITZ_TCP_PORT,
                       user=FRITZ_USERNAME,
                       password=''):
    fi = FritzInspection(address, port, user, password)
    fi.view_header()
    fi.view_servicenames()


def print_api(address=FRITZ_IP_ADDRESS,
              port=FRITZ_TCP_PORT,
              user=FRITZ_USERNAME,
              password=''):
    fi = FritzInspection(address, port, user, password)
    fi.view_header()
    fi.view_complete()


# ---------------------------------------------------------
# cli-section:
# ---------------------------------------------------------

def get_cli_arguments():
    parser = argparse.ArgumentParser(description='FritzBox API')
    parser.add_argument('-i', '--ip-address',
                        nargs='?', default=None, const=None,
                        dest='address',
                        help='Specify ip-address of the FritzBox to connect to.'
                             'Default: %s' % FRITZ_IP_ADDRESS)
    parser.add_argument('--port',
                        nargs='?', default=None, const=None,
                        help='Port of the FritzBox to connect to. '
                             'Default: %s' % FRITZ_TCP_PORT)
    parser.add_argument('-u', '--username',
                        nargs='?', default=os.getenv('FRITZ_USERNAME', None),
                        help='Fritzbox authentication username')
    parser.add_argument('-p', '--password',
                        nargs='?', default=os.getenv('FRITZ_PASSWORD', None),
                        help='Fritzbox authentication password')
    parser.add_argument('-r', '--reconnect',
                        action='store_true',
                        help='Reconnect and get a new ip')
    parser.add_argument('-s', '--services',
                        action='store_true',
                        help='List all available services')
    parser.add_argument('-S', '--serviceactions',
                        nargs=1,
                        help='List actions for the given service: <service>')
    parser.add_argument('-a', '--servicearguments',
                        nargs=1,
                        help='List arguments for the actions of a '
                             'specified service: <service>.')
    parser.add_argument('-A', '--actionarguments',
                        nargs=2,
                        help='List arguments for the given action of a '
                             'specified service: <service> <action>.')
    parser.add_argument('-c', '--complete',
                        action='store_true',
                        help='List all services with actionnames and arguments.'
                        )
    args = parser.parse_args()
    return args

def main():
    args = get_cli_arguments()
    fi = FritzInspection(args.address, args.port, args.username, args.password)
    fi.view_header()
    if args.services:
        fi.view_servicenames()
    elif args.serviceactions:
        fi.view_actionnames(args.serviceactions[0])
    elif args.servicearguments:
        fi.view_servicearguments(args.servicearguments[0])
    elif args.actionarguments:
        fi.view_actionarguments(args.actionarguments[0],
                                args.actionarguments[1])
    elif args.complete:
        fi.view_complete()
    elif args.reconnect:
        fi.fc.reconnect()
    print()  # print an empty line

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
Utility module for FritzConnection to list & switch the known callforwardings.

License: MIT https://opensource.org/licenses/MIT
Author: Julian Tatsch
"""

import os
import argparse
from lxml import etree

# tiny hack to run this as a package but also from the command line. In
# the latter case ValueError is raised from python 2.7 and SystemError
# from Python 3.5
try:
    from . import fritzconnection
except (ValueError, SystemError, ImportError):
    import fritzconnection

__version__ = '0.6.4_1'

SERVICE = 'X_AVM-DE_OnTel'


def get_version():
    """Get the version of the module."""
    return __version__


class FritzCallforwarding(object):
    """The FritzCallForwarding class to manage call forwardings.

    It allows to list and switch the known call forwardings of a fritzbox
    """

    def __init__(self,
                 fritz_connection=None,
                 address=fritzconnection.FRITZ_IP_ADDRESS,
                 port=fritzconnection.FRITZ_TCP_PORT,
                 user=fritzconnection.FRITZ_USERNAME,
                 password=''):
        """Initialize a FritzCallforwarding instance."""
        super(FritzCallforwarding, self).__init__()
        if fritz_connection is None:
            fritz_connection = fritzconnection.FritzConnection(address, port,
                                                               user, password)
        self.fritz_connection = fritz_connection
        self.service = 1

    def action(self, actionname, **kwargs):
        """Perform an action on a service."""
        return self.fritz_connection.call_action(SERVICE+':'+str(self.service),
                                                 actionname, **kwargs)

    @property
    def modelname(self):
        """Get the modelname from the fritzbox."""
        return self.fritz_connection.modelname

    def count_forwardings(self):
        """Get the total number of call forwardings from the fritzbox."""
        return self.action('GetNumberOfDeflections')['NewNumberOfDeflections']

    def get_call_forwardings(self, filter_blocked=True):
        """Get the forwardings list from the fritzbox as unparsed xml blob."""
        raw_deflections = self.action('GetDeflections')['NewDeflectionList']
        return self.parse_call_forwardings(raw_deflections, filter_blocked)

    def get_call_forwarding_by_uid(self, uid):
        """Get a single call forwarding dict for a uid.

        Valid dict-keys are:
        'uid', 'from_number', 'to_number', 'connection_type', 'enabled'
        """
        kwargs = {'NewDeflectionId': uid}
        deflection_dict = self.action('GetDeflection', **kwargs)
        return {'uid': uid,
                'from_number': deflection_dict['NewNumber'],
                'to_number': deflection_dict['NewDeflectionToNumber'],
                'connection_type': deflection_dict['NewType'],
                'enabled': int(deflection_dict['NewEnable'])}

    def get_call_forwarding_status_by_uid(self, uid):
        """Get the call forwarding status for a uid."""
        return self.get_call_forwarding_by_uid(uid)['enabled']

    def set_call_forwarding(self, uid, enable):
        """Enable call forwarding on the fritzbox for a uid."""
        kwargs = {'NewDeflectionId': uid, 'NewEnable': enable}
        self.action('SetDeflectionEnable', **kwargs)
        return self.get_call_forwarding_status_by_uid(uid)

    @staticmethod
    def parse_call_forwardings(raw_call_forwardings, filter_blocked=True):
        """
        Parse and filter call forwardings xml blob.

        Optionally filter blocked numbers out.
        """
        call_forwardings = []
        element_tree = etree.fromstring(raw_call_forwardings)
        for element in element_tree.findall('Item'):
            uid = element.find('DeflectionId').text
            enabled = int(element.find('Enable').text)
            connection_type = element.find('Type').text
            from_number = element.find('Number').text
            to_number = element.find('DeflectionToNumber').text
            # forwardings without to_number are blocked numbers
            if filter_blocked and to_number is None:
                continue

            call_forwardings.append({'uid': uid,
                                     'from_number': from_number,
                                     'to_number': to_number,
                                     'connection_type': connection_type,
                                     'enabled': enabled})
        return call_forwardings

# ---------------------------------------------------------
# terminal-output:
# ---------------------------------------------------------


def _print_header(call_forwarding):
    print('\FritzCallforwarding:')
    print('{:<30}{}'.format('version:', get_version()))
    print('{:<30}{}'.format('model:', call_forwarding.modelname))
    print('{:<30}{}'.format('ip:', call_forwarding.fritz_connection.address))


def print_callforwardings(call_forwarding):
    """Print a nice table with all call forwardings."""
    print('\n{}:{}\n'.format(SERVICE, call_forwarding.service))
    print('{:>5} {:<15} {:<15} {:<10} {:<9}\n'.format(
        'index', 'from', 'to', 'type', 'status'))
    for call_forwarding_entry in call_forwarding.get_call_forwardings():
        status = 'active' if call_forwarding_entry['enabled'] == 1 else 'disabled'
        to_number = '-' if call_forwarding_entry['to_number'] is None else call_forwarding_entry['to_number']
        print('{:>5} {:<15} {:<15} {:<10} {:<9}'.format(
            call_forwarding_entry['uid'],
            call_forwarding_entry['from_number'],
            to_number,
            call_forwarding_entry['connection_type'],
            status
            ))


def _print_detail(call_forwarding, detail, quiet):
    """Print the details of a call forwarding entry."""
    uid = detail[0].lower()
    info = call_forwarding.get_call_forwarding_by_uid(uid)
    if info:
        if not quiet:
            print('\n{:<30}{}'.format('Details for index:', uid))
            print('{:<30}{}:{}{}\n'.format('', SERVICE, call_forwarding.service,
                                           call_forwarding.count_forwardings))
            for key, value in info.items():
                print('{:<30}: {}'.format(key, value))
        else:
            print(info['enabled'])
    else:
        if quiet:
            print('0')


def _print_switch(call_forwarding, switch, quiet):
    """Switch and print the details of a call forwardings entry."""
    uid = switch[0].lower()
    if switch[1].lower() in ['on', 'enable', 'enabled', 'active']:
        call_forwarding.set_call_forwarding(uid, 1)
    else:
        call_forwarding.set_call_forwarding(uid, 0)
    _print_detail(call_forwarding, uid, quiet)


def _print_nums(call_forwarding):
    print('{}: {}'.format(SERVICE+':'+str(call_forwarding.service),
                          call_forwarding.count_forwardings()))

# ---------------------------------------------------------
# cli-section:
# ---------------------------------------------------------


def _get_cli_arguments():
    parser = argparse.ArgumentParser(description='FritzBox Callforwarding')
    parser.add_argument('-i', '--ip-address',
                        nargs='?', default=None, const=None,
                        dest='address',
                        help='ip-address of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_IP_ADDRESS)
    parser.add_argument('--port',
                        nargs='?', default=None, const=None,
                        dest='port',
                        help='port of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_TCP_PORT)
    parser.add_argument('-u', '--username',
                        nargs='?', default=None, const=None,
                        help='Fritzbox authentication username')
    parser.add_argument('-p', '--password', required=True,
                        nargs='?', default=None, const=None,
                        help='Fritzbox authentication password')
    parser.add_argument('-a', '--all',
                        action='store_true',
                        help='Show all forwardings '
                             '(default if no other options given)')
    parser.add_argument('-n', '--nums',
                        action='store_true',
                        help='Show number of call forwardings')
    parser.add_argument('-d', '--detail',
                        nargs=1, default='',
                        help='Show information about call forwarding at index'
                             '(DETAIL: index)')
    parser.add_argument('-q', '--quiet',
                        action='store_true',
                        help='Quiet mode '
                             '(just return state as 0|1 for a requested index)')
    parser.add_argument('-s', '--switch',
                        nargs=2, default='',
                        help='Enable call forwarding at index'
                        '(e.g. 1 enable)')
    args = parser.parse_args()
    return args


def _print_status(arguments):
    call_forwarding = FritzCallforwarding(address=arguments.address,
                                          port=arguments.port,
                                          user=arguments.username,
                                          password=arguments.password)

    if not arguments.quiet:
        _print_header(call_forwarding)

    if arguments.detail:
        _print_detail(call_forwarding, arguments.detail, arguments.quiet)
    elif arguments.switch:
        _print_switch(call_forwarding, arguments.switch, arguments.quiet)
    elif arguments.nums:
        _print_nums(call_forwarding)
    else:
        print_callforwardings(call_forwarding)


def main():
    """Run the module on its own."""
    _print_status(_get_cli_arguments())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test.py

unittests for fritzconnection

The xml-test-content is loaded from files because we need well defined
values which may vary by different AVM FritzBox models.
"""

import os.path
import unittest

from fritzconnection import (
    FritzXmlParser,
    FritzDescParser,
    FritzSCDPParser,
    FritzAction,
    FritzActionArgument,
    FritzService,
    )

FRITZBOX_MODEL = 'FRITZ!Box Fon WLAN 7170'
DESCRIPTION_FILE = os.path.join(os.path.dirname(__file__),
                                'test_xml', 'igddesc.xml')
SCPD_FILE = os.path.join(os.path.dirname(__file__),
                         'test_xml', 'igdconnSCPD.xml')


class TestFritzXmlParser(unittest.TestCase):

    def setUp(self):
        self.fp = FritzXmlParser(None, None, filename=DESCRIPTION_FILE)
        self.namespace = 'urn:schemas-upnp-org:device-1-0'

    def test_namespace(self):
        self.assertEqual(self.namespace, self.fp.namespace)

    def test_nodename(self):
        self.assertEqual('{%s}%s' % (self.namespace, 'mynode'),
                          self.fp.nodename('mynode'))


class TestFritzDescParser(unittest.TestCase):

    def setUp(self):
        self.fp = FritzDescParser(None, None, filename=DESCRIPTION_FILE)

    def test_modelname(self):
        self.assertEqual(FRITZBOX_MODEL, self.fp.get_modelname())

    def test_getservices(self):
        """
        We get a list of services. Instead of comparing the entire list
        we look at the last item of the first tuple in the list, which
        should be '/any.xml'. If this test passes we assume that this is
        the case for all tuples in the list.
        """
        services = self.fp.get_services()
        self.assertEqual(r'/any.xml', services[0].scpd_url)


class FritzSCDPTestParser(FritzSCDPParser):
    """Helper subclass of FritzSCDPParser to generate an object configured
       for testing purposes
    """

    def __init__(self, *args, **kwargs):
        self.service = FritzService(
                    'urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1',
                    '/upnp/control/WANCommonIFC1',
                    '/igdicfgSCPD.xml'
                    )
        super(FritzSCDPTestParser, self).__init__(None, None, self.service, filename=SCPD_FILE)


class TestFritzSCDPParser(unittest.TestCase):

    def setUp(self):
        self.fp = FritzSCDPTestParser()

    def test_get_service_name(self):
        self.assertEqual('WANCommonInterfaceConfig:1', self.fp.service.name)

    def test_read_state_variables(self):
        """Parse the stateVariables and check for a single value."""
        self.fp._read_state_variables()
        self.assertEqual(
            'string', self.fp.state_variables.get('ExternalIPAddress'))

    def test_get_actions(self):
        """Read actionnames and corresponding arguments from xml-file."""
        actions = self.fp.get_actions({})
        first_action = actions[0]
        self.assertEqual(
            type(FritzAction(None, None, {})), type(first_action))


class TestFritzAction(unittest.TestCase):
    """Test for response parsing."""

    # The action we use for testing:
    action_name = 'GetStatusInfo'

    # service_type for simulated call_action('GetStatusInfo'):
    service_type = 'urn:schemas-upnp-org:service:WANIPConnection:1'

    # the control_url for this action (for object initialisation)
    control_url = '/upnp/control/WANIPConn1'

    # typical response as given by call_action('GetStatusInfo'):
    response = b'<?xml version="1.0" encoding="utf-8"?>\n<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">\n<s:Body>\n<u:GetStatusInfoResponse xmlns:u="urn:schemas-upnp-org:service:WANIPConnection:1">\n<NewConnectionStatus>Connected</NewConnectionStatus>\n<NewLastConnectionError>ERROR_NONE</NewLastConnectionError>\n<NewUptime>29938</NewUptime>\n</u:GetStatusInfoResponse>\n</s:Body>\n</s:Envelope>'

    def get_arguments(self):
        """
        We need the 'GetStatusInfo' arguments for parsing the response.
        So we extract them here. Because this is code tested before this
        function gets called, we know it works.
        Returns a dictionary with argumentname,
        FritzActionArgument-object pairs.
        """
        scdp = FritzSCDPTestParser()
        actions = {action.name: action for action in scdp.get_actions({})}
        return actions[self.action_name].arguments

    def test_parse_response(self):
        fa = FritzAction(self.service_type, self.control_url, {})
        fa.arguments = self.get_arguments()  # argument-injection
        result = fa.parse_response(self.response)
        self.assertEqual('Connected', result['NewConnectionStatus'])
        self.assertEqual('ERROR_NONE', result['NewLastConnectionError'])
        self.assertEqual(29938,  result['NewUptime'])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# new since 0.6.5
package_version = '0.8.1'

from .fritzconnection import (
    FritzConnection,
    FritzInspection,
    print_servicenames,
    print_api,
)
from . fritzcall import FritzCall
from . fritzcallforwarding import (
    FritzCallforwarding,
    print_callforwardings,
)
from .fritzhosts import (
    FritzHosts,
    print_hosts,
)
from .fritzphonebook import (
    FritzPhonebook,
    print_phonebooks,
)
from .fritzstatus import (
    FritzStatus,
    print_status,
)
from .fritzwlan import FritzWLAN



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
fritzstatus.py

Modul to read status-informations from an AVM FritzBox.

License: MIT https://opensource.org/licenses/MIT
Source: https://bitbucket.org/kbr/fritzconnection
Author: Klaus Bremer
"""

__version__ = '0.8.1'

import argparse
import collections
import os
import time

# tiny hack to run this as a package but also from the command line. In
# the latter case ValueError is raised from python 2.7 and SystemError
# from Python 3.5 and ImportError from Python 3.6
try:
    from . import fritzconnection
    from . import fritztools
except (ValueError, SystemError, ImportError):
    import fritzconnection
    import fritztools


# version-access:
def get_version():
    return __version__


class FritzStatus(object):
    """
    Class for requesting status-informations:
    up, down, ip, activity (bytes per second send/received)
    Every property will raise an IOError if the connection
    with the FritzBox fails.

    Keep in mind, that FritzBoxes may return different informations
    about the status depending whether this service gets called with or
    without a password.
    """

    def __init__(self, fc=None, address=None, port=None,
                       user=None, password=None):
        super(FritzStatus, self).__init__()
        if fc is None:
            fc = fritzconnection.FritzConnection(
                address=address,
                port=port,
                user=user,
                password=password,
            )
        self.fc = fc
        self.last_bytes_sent = self.bytes_sent
        self.last_bytes_received = self.bytes_received
        self.last_traffic_call = time.time()

    @property
    def modelname(self):
        return self.fc.modelname

    @property
    def is_linked(self):
        """Returns True if the FritzBox is physically linked to the provider."""
        status = self.fc.call_action('WANCommonIFC',
                                     'GetCommonLinkProperties')
        return status['NewPhysicalLinkStatus'] == 'Up'

    @property
    def is_connected(self):
        """
        Returns True if the FritzBox has established an internet-connection.
        """
        status = self.fc.call_action('WANIPConn', 'GetStatusInfo')
        return status['NewConnectionStatus'] == 'Connected'

    @property
    def wan_access_type(self):
        """Returns connection-type: DSL, Cable."""
        return self.fc.call_action('WANCommonIFC',
            'GetCommonLinkProperties')['NewWANAccessType']

    @property
    def external_ip(self):
        """Returns the external ip-address."""
        return self.fc.call_action('WANIPConn',
            'GetExternalIPAddress')['NewExternalIPAddress']

    @property
    def external_ipv6(self):
        """Returns the external ip-address."""
        return self.fc.call_action('WANIPConn',
            'X_AVM_DE_GetExternalIPv6Address')['NewExternalIPv6Address']

    @property
    def uptime(self):
        """uptime in seconds."""
        status = self.fc.call_action('WANIPConn', 'GetStatusInfo')
        return status['NewUptime']

    @property
    def str_uptime(self):
        """uptime in human readable format."""
        mins, secs = divmod(self.uptime, 60)
        hours, mins = divmod(mins, 60)
        return '%02d:%02d:%02d' % (hours, mins, secs)

    @property
    def bytes_sent(self):
        status = self.fc.call_action('WANCommonIFC',
                                     'GetTotalBytesSent')
        return status['NewTotalBytesSent']

    @property
    def bytes_received(self):
        status = self.fc.call_action('WANCommonIFC',
                                     'GetTotalBytesReceived')
        return status['NewTotalBytesReceived']

    @property
    def transmission_rate(self):
        """
        Returns the upstream, downstream values as a tuple in bytes per
        second. Use this for periodical calling.
        """
        sent = self.bytes_sent
        received = self.bytes_received
        traffic_call = time.time()
        time_delta = traffic_call - self.last_traffic_call
        upstream = int(1.0 * (sent - self.last_bytes_sent)/time_delta)
        downstream = int(1.0 * (received - self.last_bytes_received)/time_delta)
        self.last_bytes_sent = sent
        self.last_bytes_received = received
        self.last_traffic_call = traffic_call
        return upstream, downstream

    @property
    def str_transmission_rate(self):
        """Returns a tuple of human readable transmission rates in bytes."""
        upstream, downstream = self.transmission_rate
        return (
            fritztools.format_num(upstream),
            fritztools.format_num(downstream)
        )

    @property
    def max_linked_bit_rate(self):
        """
        Returns a tuple with the maximun upstream- and downstream-rate
        of the physical link. The rate is given in bits/sec.
        """
        return self._get_max_bit_rate('WANCommonInterfaceConfig')

    @property
    def max_bit_rate(self):
        """
        Returns a tuple with the maximun upstream- and downstream-rate
        of the given connection. The rate is given in bits/sec.
        """
        return self._get_max_bit_rate('WANCommonIFC')

    def _get_max_bit_rate(self, servicename):
        """
        internal method to get the upstream and downstream-rates for
        different services of the WANCommonInterfaceConfig1 ServiceType.
        """
        status = self.fc.call_action(servicename, 'GetCommonLinkProperties')
        downstream = status['NewLayer1DownstreamMaxBitRate']
        upstream = status['NewLayer1UpstreamMaxBitRate']
        return upstream, downstream

    @property
    def max_byte_rate(self):
        """
        Same as max_bit_rate but returns the rate in bytes/sec.
        """
        upstream, downstream = self.max_bit_rate
        return upstream / 8.0, downstream / 8.0

    @property
    def str_max_linked_bit_rate(self):
        """
        Returns a human readable maximun upstream- and downstream-rate
        of the given connection. The rate is given in bits/sec.
        """
        upstream, downstream = self.max_linked_bit_rate
        return (
            fritztools.format_rate(upstream, unit='bits'),
            fritztools.format_rate(downstream, unit ='bits')
        )

    @property
    def str_max_bit_rate(self):
        """
        Returns a human readable maximun upstream- and downstream-rate
        of the given connection. The rate is given in bits/sec.
        """
        upstream, downstream = self.max_bit_rate
        return (
            fritztools.format_rate(upstream, unit='bits'),
            fritztools.format_rate(downstream, unit ='bits')
        )

    def reconnect(self):
        """Makes a reconnection with a new external ip."""
        self.fc.reconnect()


# ---------------------------------------------------------
# terminal-output:
# ---------------------------------------------------------

def print_status(address=None, port=None, user=None, password=None):
    print('\nFritzStatus:')
    print('{:<22}{}'.format('version:', get_version()))
    fs = FritzStatus(address=address, port=port, user=user, password=password)
    status_informations = collections.OrderedDict([
        ('model:', fs.modelname),
        ('is linked:', fs.is_linked),
        ('is connected:', fs.is_connected),
        ('external ip:', fs.external_ip),
        ('uptime:', fs.str_uptime),
        ('bytes send:', fs.bytes_sent),
        ('bytes received:', fs.bytes_received),
        ('max. bit rate:', fs.str_max_bit_rate),
        ])
    try:
        information = fs.str_max_linked_bit_rate
    except fritzconnection.ServiceError:
        information = 'password required for information'
    except fritzconnection.AuthorizationError as err:
        information = str(err)
    status_informations['max. linked bit rate:'] = information
    for status, information in status_informations.items():
        print('{:<22}{}'.format(status, information))


# ---------------------------------------------------------
# cli-section:
# ---------------------------------------------------------

def _get_cli_arguments():
    parser = argparse.ArgumentParser(description='FritzBox Status')
    parser.add_argument('-i', '--ip-address',
                        nargs='?', default=None, const=None,
                        dest='address',
                        help='ip-address of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_IP_ADDRESS)
    parser.add_argument('-u', '--username',
                        nargs='?', default=None, const=None,
                        help='Fritzbox authentication username')
    parser.add_argument('-p', '--password',
                        nargs='?', default=None, const=None,
                        help='Fritzbox authentication password')
    parser.add_argument('--port',
                        nargs='?', default=None, const=None,
                        dest='port',
                        help='port of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_TCP_PORT)
    args = parser.parse_args()
    return args


def _print_status(arguments):
    print_status(
        address=arguments.address,
        port=arguments.port,
        user=arguments.username,
        password=arguments.password,
    )

def main():
    _print_status(_get_cli_arguments())

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
fritzmonitor.py

Implement a tkinter-based grafic interface to view basic status- and
traffic-informations.

License: MIT https://opensource.org/licenses/MIT
Source: https://bitbucket.org/kbr/fritzconnection
Author: Klaus Bremer
"""

__version__ = '0.7.3'

import argparse
try:
    # python 2
    import Tkinter as tk
    import tkFont as tkfont
except ImportError:
    # python 3
    import tkinter as tk
    import tkinter.font as tkfont
try:
    from . import fritzconnection
    from . import fritzstatus
    from . import fritztools
except (ValueError, SystemError, ImportError):
    import fritzconnection
    import fritzstatus
    import fritztools


class MeterRectangle(object):
    """
    a Tkinter meter-rectangel. This is a rectangle with a
    background-color that can be filled up from left to right with
    another color (fill).
    This object must have a canvas-widget as parent.
    """

    def __init__(self, canvas, xpos, ypos, meter_width, meter_height,
                 fill="#0080FF", background="#E6E6E6", horizontal=True,
                 **kwargs):
        self.canvas = canvas
        self.xpos = xpos
        self.ypos = ypos
        self.width = meter_width
        self.height = meter_height
        self.horizontal = horizontal
        self.container = canvas.create_rectangle(xpos, ypos,
                                             xpos + meter_width,
                                             ypos + meter_height,
                                             fill = background,
                                             outline = background,
                                             **kwargs)
        self.meter = canvas.create_rectangle(xpos, ypos,
                                             xpos,
                                             ypos + meter_height,
                                             fill = fill,
                                             outline = fill,
                                             **kwargs)

    def set_fraction(self, value):
        """Set the meter indicator. Value should be between 0 and 1."""
        if value < 0:
            value *= -1
        value = min(value, 1)
        if self.horizontal:
            width = int(self.width * value)
            height = self.height
        else:
            width = self.width
            height = int(self.height * value)
        self.canvas.coords(self.meter, self.xpos, self.ypos,
                           self.xpos + width, self.ypos + height)


class FritzMonitor(tk.Frame):

    def __init__(self, master=None,
                       address=fritzconnection.FRITZ_IP_ADDRESS,
                       port=fritzconnection.FRITZ_TCP_PORT):
        tk.Frame.__init__(self, master)
        self.status = fritzstatus.FritzStatus(address=address, port=port)
        self.max_upstream, self.max_downstream = self.status.max_byte_rate
        self.max_stream_rate = tk.StringVar()
        self.connection_state = tk.StringVar()
        self.ip = tk.StringVar()
        self.uptime = tk.StringVar()
        self.traffic_info = tk.StringVar()
        self.bytes_received = self.status.bytes_received
        self.bytes_sent = self.status.bytes_sent
        self.grid()
        self.create_widgets()
        self.update_status()

    def get_stream_rate_str(self):
        up, down = self.status.str_max_bit_rate
        return "Down: %s, Up: %s" % (down, up)

    def update_connection_status(self):
        if self.status.is_connected:
            color, text = 'green', 'up'
        else:
            color, text = 'red', 'down'
        self.connection_label.config(fg=color)
        self.connection_state.set(text)

    def update_traffic_info(self):
        received = fritztools.format_num(
            self.status.bytes_received - self.bytes_received)
        sent = fritztools.format_num(
            self.status.bytes_sent - self.bytes_sent)
        text = "received: %s, send: %s" % (received, sent)
        self.traffic_info.set(text)

    def update_status(self):
        """Update status informations in tkinter window."""
        try:
            # all this may fail if the connection to the fritzbox is down
            self.update_connection_status()
            self.max_stream_rate.set(self.get_stream_rate_str())
            self.ip.set(self.status.external_ip)
            self.uptime.set(self.status.str_uptime)
            upstream, downstream = self.status.transmission_rate
        except IOError:
            # here we inform the user about being unable to
            # update the status informations
            pass
        else:
            # max_downstream and max_upstream may be zero if the
            # fritzbox is configured as ip-client.
            if self.max_downstream > 0:
                self.in_meter.set_fraction(
                    1.0 * downstream / self.max_downstream)
            if self.max_upstream > 0:
                self.out_meter.set_fraction(1.0 * upstream / self.max_upstream)
            self.update_traffic_info()
        self.after(1000, self.update_status)

    def create_widgets(self):
        self.place_header()
        self.place_meter()
        self.place_traffic_info()
        self.place_connection_info()
        self.place_buttons()

    def place_header(self):
        tk.Label(self, text="%s:" % self.status.modelname).grid(
            row=0, column=0, sticky=tk.NW, padx=5)
        self.connection_label = tk.Label(self,
            textvariable=self.connection_state, fg='red')
        self.connection_label.grid(row=0, column=1)
        tk.Label(self, textvariable=self.max_stream_rate).grid(
            row=1, column=0, sticky=tk.W, padx=5)

    def place_traffic_info(self):
        tk.Label(self, textvariable=self.traffic_info,
                       font=tkfont.Font(family='courier', size=12),
                       fg='grey',
                ).grid(row=2, column=0, columnspan=2,
                       padx=5, pady=5, sticky=tk.W)

    def place_meter(self):
        pane = tk.Canvas(self, height=50)
        pane.grid(row=3, column=0, columnspan=2, sticky=tk.NW, pady=10)
        self.in_meter = MeterRectangle(pane, 10, 10, 240, 12)
        self.out_meter = MeterRectangle(
            pane, 10, 30, 240, 12, fill="#FF6666")

    def place_connection_info(self):
        tk.Label(self, textvariable=self.ip).grid(
            row=4, column=0, sticky=tk.W, padx=5)
        tk.Label(self, textvariable=self.uptime).grid(row=4, column=1)

    def place_buttons(self):
        tk.Button(self, text='Reconnect', command=self.status.reconnect).grid(
            row=5, column=0, sticky=tk.W, padx=10)
        tk.Button(self, text='Quit', command=self.quit).grid(row=5, column=1)


# ---------------------------------------------------------
# cli-section:
# ---------------------------------------------------------

def _get_cli_arguments():
    parser = argparse.ArgumentParser(description='FritzBox Monitor')
    parser.add_argument('-i', '--ip-address',
                        nargs='?', default=fritzconnection.FRITZ_IP_ADDRESS,
                        dest='address',
                        help='ip-address of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_IP_ADDRESS)
    parser.add_argument('-p', '--port',
                        nargs='?', default=fritzconnection.FRITZ_TCP_PORT,
                        dest='port',
                        help='port of the FritzBox to connect to. '
                             'Default: %s' % fritzconnection.FRITZ_TCP_PORT)
    args = parser.parse_args()
    return args

def main():
    arguments = _get_cli_arguments()
    app = FritzMonitor(address=arguments.address, port=arguments.port)
    app.master.title('FritzMonitor')
    app.mainloop()

if __name__ == '__main__':
    main()
